﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Data;
using System.Data.Entity;
using System.Linq;
using System.Net;
using System.Web;
using System.Web.Mvc;
using StartToBike.DAL;
using StartToBike.Models;
using StartToBike.ViewModels;

namespace StartToBike.Controllers
{
    public class ToursController : Controller
    {
        private DatabaseContext db = new DatabaseContext();

        // Overview of tours
        public ActionResult Index()
        {
            // Get the user
            User user = (User)Session["user"];
            
            if (user == null)
            {
                return new HttpStatusCodeResult(HttpStatusCode.BadRequest);
            }

            // Get list of tours where user is owner of or participates in
            user = db.Users.Find(user.ID);
            List<Tour> ownerTours = user.OwnerOfTours.ToList();
            List<Tour> participatesTours = user.ParticipatesInTours.ToList();
            List<Tour> allTours = ownerTours.Concat(participatesTours).ToList();

            return View(allTours);
        }

        // A specific tour
        public ActionResult Details(int? id)
        {
            if (id == null)
            {
                return new HttpStatusCodeResult(HttpStatusCode.BadRequest);
            }
            Tour tour = db.Tours.Find(id);
            if (tour == null)
            {
                return HttpNotFound();
            }
            return View(tour);
        }

        // New tour
        public ActionResult Create()
        {
            ViewBag.RouteID = new SelectList(db.Routes, "ID", "Name");
            return View();
        }

        // New tour (process data)
        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult Create([Bind(Include = "ID,UserID,Description,Name,Start,RouteID")] Tour tour)
        {
            User owner = (User)Session["user"];
            tour.UserID = owner.ID;
            if (ModelState.IsValid)
            {
                db.Tours.Add(tour);
                db.SaveChanges();
                return RedirectToAction("Index");
            }

            ViewBag.RouteID = new SelectList(db.Routes, "ID", "Name", tour.RouteID);
            return View(tour);
        }

        // New invite
        public ActionResult Invite(int tourID)
        {
            // Check if tour is set
            Tour tour = db.Tours.Find(tourID);
            if (tour == null)
            {
                return new HttpStatusCodeResult(HttpStatusCode.BadRequest);
            }

            // Create a new invite
            Invite invite = tour.CreateInvite();

            // Save changes to database if modelstate is valid
            if (ModelState.IsValid)
            {
                db.Invites.Add(invite);
                db.SaveChanges();
            }

            // Return the view
            return View(invite);
        }

        // Join a tour
        public ActionResult Join()
        {
            return View();
        }

        // Join a tour (process data)
        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult Join([Bind(Include = "ID,Link,TourID,Tour")] Invite invite)
        {
            // Get the tour where the invite is for
            invite = db.Invites.Where(i => i.Link == invite.Link).FirstOrDefault();
            Tour tour = db.Tours.Where(t => t.ID == invite.TourID).Include(t => t.Participants).FirstOrDefault();

            // Join the tour
            User user = (User)Session["user"];
            user = db.Users.Find(user.ID);

            if (!tour.Join(user))
            {
                return View("Details", tour);
            }

            // Save changes
            if (ModelState.IsValid)
            {
                db.SaveChanges();
            }

            // Return view
            return View("Details", tour);
        }

        // Edit a tour
        public ActionResult Edit(int? id)
        {
            // Check if id is set
            if (id == null)
            {
                return new HttpStatusCodeResult(HttpStatusCode.BadRequest);
            }

            // Get the tour
            Tour tour = db.Tours.Find(id);

            // Check if tour exists
            if (tour == null)
            {
                return HttpNotFound();
            }

            return View(tour);
        }

        // Edit a tour (process data)
        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult Edit([Bind(Include = "ID,UserID,Description,Name,Start,RouteID")] Tour tour)
        {
            // Check if modelstate is valid and if so save changes
            if (ModelState.IsValid)
            {
                db.Entry(tour).State = EntityState.Modified;
                db.SaveChanges();
                return RedirectToAction("Details", tour);
            }

            return View(tour);
        }

        // Leave a tour
        public ActionResult Leave(int tourID)
        {
            // Get the tour and user
            Tour tour = db.Tours.Find(tourID);
            User user = (User)Session["user"];
            user = db.Users.Find(user.ID);

            // Remove the participant
            tour.Participants.Remove(user);

            // Save changes
            if (ModelState.IsValid)
            {
                db.SaveChanges();
            }
            
            // Redirect to overview of tours
            return RedirectToAction("Index");
        }

        // Delete a tour
        public ActionResult Delete(int tourID)
        {
            // Get the tour
            Tour tour = db.Tours.Find(tourID);

            // Remove the tour
            db.Tours.Remove(tour);

            // Save changes
            if (ModelState.IsValid)
            {
                db.SaveChanges();
            }

            // Redirect to overview of tours
            return RedirectToAction("Index");
        }

        // Chat
        public ActionResult Chat(int tourID)
        {
            // Get the tour and chats
            Tour tour = db.Tours.Find(tourID);
            List<Chat> chats = tour.Chats.ToList();

            // Fill ViewModel
            TourChatViewModel tourChat = new TourChatViewModel();
            tourChat.Chats = chats;
            tourChat.Tour = tour;

            // Return view
            return View(tourChat);
        }

        // Send chatmessage
        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult Chat([Bind(Include = "Tour,Chats,NewChat")] TourChatViewModel tourChat)
        {
            // Get the chat
            Chat chat = tourChat.NewChat;

            // Get the user
            User user = (User)Session["user"];
            user = db.Users.Find(user.ID);

            // Set properties of chat
            chat.UserID = user.ID;
            chat.SendDateTime = DateTime.Now;

            // Get the tour
            Tour tour = db.Tours.Find(tourChat.Tour.ID);
            tour.SendChat(chat);

            // Add chat to database
            if (ModelState.IsValid)
            {
                db.SaveChanges();
            }

            // Return to chat
            return RedirectToAction("Chat", new { tourID = tour.ID });
        }

        // Select a route
        public ActionResult SelectRoute(int tourID)
        {
            Tour tour = db.Tours.Find(tourID);

            List<Route> routes = db.Routes.ToList();
            ViewData["routeList"] = routes.Select(x => new SelectListItem { Value = x.ID.ToString(), Text = x.Name });
            
            return View(tour);
        }

        // Select a route (process data)
        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult SelectRoute([Bind(Include = "ID,UserID,Description,Name,Start,RouteID")] Tour editedTour)
        {
            Tour tour = db.Tours.Find(editedTour.ID);
            tour.RouteID = editedTour.RouteID;
            
            // Check if modelstate is valid and if so save changes
            if (ModelState.IsValid)
            {
                db.Entry(tour).State = EntityState.Modified;
                db.SaveChanges();
                return RedirectToAction("Details", tour);
            }

            return View(tour);
        }

        protected override void Dispose(bool disposing)
        {
            if (disposing)
            {
                db.Dispose();
            }
            base.Dispose(disposing);
        }
    }
}
