﻿using StartToBike.DAL;
using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.ComponentModel.DataAnnotations.Schema;
using System.Linq;
using System.Web;

namespace StartToBike.Models
{
    /// <summary>
    /// A tour which multiple people can join to cycle together.
    /// </summary>
    public class Tour
    {
        private DatabaseContext db = new DatabaseContext();

        /// <summary>
        /// Identifier.
        /// </summary>
        public int ID { get; set; }

        /// <summary>
        /// ID of the owner of the tour.
        /// </summary>
        [ForeignKey("Owner")]
        public int? UserID { get; set; }

        /// <summary>
        /// Owner of the tour.
        /// </summary>
        public virtual User Owner { get; set; }

        /// <summary>
        /// Description of the tour.
        /// </summary>
        [Display(Name = "Omschrijving")]
        public string Description { get; set; }

        /// <summary>
        /// Name of the tour.
        /// </summary>
        [Display(Name = "Naam")]
        public string Name { get; set; }

        /// <summary>
        /// Startdate and time of the tour.
        /// </summary>
        [Display(Name = "Begindatum en -tijd")]
        public DateTime Start { get; set; }

        /// <summary>
        /// ID of the route.
        /// </summary>
        public int? RouteID { get; set; }

        /// <summary>
        /// Route to be cycled.
        /// </summary>
        public virtual Route Route { get; set; }

        /// <summary>
        /// Participants of the tour.
        /// </summary>
        public virtual ICollection<User> Participants { get; set; }

        /// <summary>
        /// Invites for the tour.
        /// </summary>
        public virtual ICollection<Invite> Invites { get; set; }

        /// <summary>
        /// Chatmessages for the tour.
        /// </summary>
        public virtual ICollection<Chat> Chats { get; set; }

        /// <summary>
        /// Create a new invite for a tour
        /// </summary>
        /// <returns>The invite</returns>
        public Invite CreateInvite()
        {
            // Create a new invite-object
            Invite invite = new Invite();

            // As long as no unique link has been found, generate new ones
            bool linkAlreadyExists = true;
            string link = String.Empty;

            while (linkAlreadyExists)
            {
                // Generate a link
                link = invite.GenerateLink();
                
                // Check if link already exists
                int occurencesOfLink = db.Invites.Where(i => i.Link == link).ToList().Count;
                if (occurencesOfLink == 0)
                {
                    linkAlreadyExists = false;
                }
            }

            // Set the tour and link in the object
            invite.Link = link;
            invite.Tour = this;
            invite.TourID = this.ID;

            // Return the invite
            return invite;
        }

        /// <summary>
        /// Add a user as a participant to a tour.
        /// </summary>
        /// <param name="user">The user who joins the tour</param>
        /// <returns>True on success, false if user is already part of the tour.</returns>
        public bool Join(User user)
        {
            // Check if user is owner of tour
            if (user.ID == this.UserID)
            {
                return false;
            }

            // Check if user already participates in tour
            List<int> participantIDs = new List<int>();
            foreach (User participant in this.Participants)
            {
                participantIDs.Add(participant.ID);
            }

            if (participantIDs.Contains(user.ID))
            {
                return false;
            }

            // Add user to tour
            this.Participants.Add(user);
            return true;
        }

        /// <summary>
        /// Send a chatmessage
        /// </summary>
        /// <param name="chat">The chat to send</param>
        /// <returns>True on success</returns>
        public bool SendChat(Chat chat)
        {
            this.Chats.Add(chat);

            return true;
        }
    }
}